from .metadata_inspector import MetadataInspector
import json
from datetime import date


class EADInspector(MetadataInspector):
    """Class for inspectors that examine records in EAD-XML."""

    def __init__(self, error_lang="en") -> None:
        """
        Construct EADInspector with specific error language.

        :param error_lang: Error language for the inspections.
        :type error_lang: str, default 'en'
        """
        super().__init__(error_lang)
        self._ead_namespace: str = "urn:isbn:1-931666-22-9"
        self._cs: list = []
        self._rights_ead: list = []
        self.configuration: dict = {
            "unittitle": {
                "collection": {"inspect": True, "min_word_num": 2},
                "class": {"inspect": True, "min_word_num": 1},
                "series": {"inspect": True, "min_word_num": 1},
                "file": {"inspect": True, "min_word_num": 2},
                "item": {"inspect": True, "min_word_num": 2},
                "_": {"inspect": True, "min_word_num": 2},
            },
            "unitdate": {
                "collection": {"inspect": True},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": True},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "abstract": {
                "collection": {"inspect": True, "min_word_num": 10},
                "class": {"inspect": False, "min_word_num": 10},
                "series": {"inspect": False, "min_word_num": 10},
                "file": {"inspect": True, "min_word_num": 10},
                "item": {"inspect": True, "min_word_num": 10},
                "_": {"inspect": True, "min_word_num": 10},
            },
            "genreform": {
                "normal": [
                    "Urkunden",
                    "Siegel",
                    "Amtsbücher, Register und Grundbücher",
                    "Akten",
                    "Karten und Pläne",
                    "Plakate und Flugblätter",
                    "Drucksachen",
                    "Bilder",
                    "Handschriften",
                    "Audio-Visuelle Medien",
                    "Datenbanken",
                    "Sonstiges",
                ],
                "collection": {"inspect": True},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": True},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "dimensions": {
                "collection": {"inspect": True},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": True},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "extent": {
                "collection": {"inspect": True},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": True},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "scopecontent": {
                "collection": {"inspect": True, "min_word_num": 100},
                "class": {"inspect": False, "min_word_num": 10},
                "series": {"inspect": False, "min_word_num": 10},
                "file": {"inspect": True, "min_word_num": 10},
                "item": {"inspect": False, "min_word_num": 10},
                "_": {"inspect": True, "min_word_num": 10},
            },
            "origination": {
                "collection": {"inspect": True, "ref": True},
                "class": {"inspect": False, "ref": True},
                "series": {"inspect": False, "ref": True},
                "file": {"inspect": True, "ref": True},
                "item": {"inspect": True, "ref": True},
                "_": {"inspect": True, "ref": True},
            },
            "materialspec": {
                "collection": {"inspect": True},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": True},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "language": {
                "collection": {"inspect": True},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": True},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "digital_archival_object": {
                "collection": {"inspect": False},
                "class": {"inspect": False},
                "series": {"inspect": False},
                "file": {"inspect": False},
                "item": {"inspect": True},
                "_": {"inspect": True},
            },
            "index": {
                "collection": {"inspect": True, "ref": True, "min_num": 5},
                "class": {"inspect": False, "ref": True, "min_num": 3},
                "series": {"inspect": False, "ref": True, "min_num": 3},
                "file": {"inspect": True, "ref": True, "min_num": 3},
                "item": {"inspect": True, "ref": True, "min_num": 3},
                "_": {"inspect": True, "ref": True, "min_num": 3},
            },
            "userestrict": {
                "collection": {"inspect": True, "ref": True},
                "class": {"inspect": False, "ref": True},
                "series": {"inspect": False, "ref": True},
                "file": {"inspect": True, "ref": True},
                "item": {"inspect": True, "ref": True},
                "_": {"inspect": True, "ref": True},
            },
        }

    @property
    def ead_namespace(self) -> str:
        """Get the EAD namespace when needed for reading attributes."""
        return self._ead_namespace

    @property
    def cs(self) -> list[object]:
        """Get or set the list of EAD components. These components are examined during the inspection."""
        return self._cs

    @cs.setter
    def cs(self, cs: list) -> None:
        self._cs = cs

    @property
    def rights_ead(self) -> list:
        """Get or set the EAD metadata rights."""
        return self._rights_ead

    @rights_ead.setter
    def rights_ead(self, rights_ead: list) -> None:
        self._rights_ead = rights_ead

    @property
    def configuration(self) -> dict:
        """Get or set the configuration. The inspection is carried out based on the configuration."""
        return self._configuration

    @configuration.setter
    def configuration(self, configuration: dict) -> None:
        self._configuration = configuration

    def read_ead(self, xml_str: str) -> None:
        """
        Parse EAD-XML from a string and assign EAD components to the inspector.

        :param xml_str: String with EAD-XML syntax
        :type xml_str: str
        """
        xml_root = MetadataInspector.read_xml(xml_str)
        self.cs = [comp for comp in xml_root.iter("{*}c")]
        self.rights_ead = xml_root.find(
            ".//{*}archdesc/{*}userestrict[@type='ead']//{*}extref"
        )

    def read_ead_file(self, file_path: str) -> None:
        """
        Parse EAD-XML from a file and assign EAD components to the inspector.

        :param file_path: File path to a XML file
        :type file_path: str
        """
        xml_root = MetadataInspector.read_xml_file(file_path)
        self.cs = [comp for comp in xml_root.iter("{*}c")]
        self.rights_ead = xml_root.find(
            ".//{*}archdesc/{*}userestrict[@type='ead']//{*}extref"
        )

    def configure(self, config: dict) -> None:
        """
        Alter the default configurations of an inspector.

        :param config: Dict of configurations with the syntax of the default configurations
        :type config: dict
        """
        for key in self.configuration.keys():
            if key in config:
                self.configure_level(key, config[key])

    def configure_level(self, setting: str, change: dict | list) -> None:
        """
        Alter a specific level in the configurations of an inspector.

        :param setting: Name of the setting which should be altered
        :type setting: str
        :param change: New configuartions for the specific setting
        :type change: dict | list
        """
        for key in self.configuration[setting].keys():
            if key in change:
                self.configure_setting(setting, key, change[key])

    def configure_setting(self, setting: str, level: str, change: list | dict) -> None:
        """
        Alter a specific setting in the configurations of an inspector.

        :param setting: Name of the setting which should be altered
        :type setting: str
        :param level: Name of the level which should be altered
        :type level: str
        :param change: New configuartions for the specific setting
        :type change: dict | list
        """
        if level in ["normal"]:
            self.configuration[setting][level] = change
            return
        if isinstance(change, dict):
            for key, value in change.items():
                if key not in self.configuration[setting][level]:
                    continue
                if key in ["inspect", "ref"]:
                    self.configuration[setting][level][key] = bool(value)
                elif key in ["min_word_num", "min_num"]:
                    self.configuration[setting][level][key] = int(value)

    def config_file(self, file_path: str) -> None:
        """
        Read a configuration file and alter the default configurations of an inspector.

        :param file_path: File path to a JSON file with configurations in the required syntax
        :type file_path: str
        """
        with open(file_path, "r") as f:
            config = json.load(f)
        self.configure(config)

    def inspect(self) -> None:
        """Carry out an inspection based on the read-in EAD components."""
        self.inspections = []
        for c in self.cs:
            level: str = self.attr(c, "level")
            if level not in ["collection", "class", "series", "file", "item"]:
                level = "_"
            inspection: dict = {}
            inspection["id"] = self.inspect_id(c)
            inspection["unitid"] = self.inspect_unitid(c)
            inspection["unittitle"] = self.inspect_unittitle(c, level)
            inspection["unitdate"] = self.inspect_unitdates(c, level)
            inspection["abstract"] = self.inspect_abstract(c, level)
            inspection["genreform"] = self.inspect_genreform(c, level)
            inspection["dimensions"] = self.inspect_dimensions(c, level)
            inspection["extent"] = self.inspect_extent(c, level)
            inspection["scopecontent"] = self.inspect_scopecontent(c, level)
            inspection["origination"] = self.inspect_originations(c, level)
            inspection["materialspec"] = self.inspect_materialspec(c, level)
            inspection["language"] = self.inspect_language(c, level)
            inspection["digital_archival_object"] = self.inspect_daos(c, level)
            inspection["index"] = self.inspect_index(c, level)
            inspection["userestrict"] = self.inspect_userestrict(c, level)
            self.inspections.append(inspection)

    def inspect_id(self, c) -> str:
        """
        Inspect component ID.

        :param c: Component of an EAD record
        :type c: etree._Element
        :return: Component ID or error message if missing
        :rtype: str
        """
        return (
            self.attr(c, "id")
            if self.has_attribute(c, "id")
            else self.error.miss_info()
        )

    def inspect_unitid(self, c) -> str:
        """
        Inspect unit ID.

        :param c: Component of an EAD record
        :type c: etree._Element
        :return: Unit ID or error message if missing
        :rtype: str
        """
        unitid = c.find("{*}did/{*}unitid")
        return unitid.text if self.text(unitid) else self.error.miss_info()

    def inspect_text(self, element, config: dict) -> list | None:
        """
        Inspect a text element.

        :param element: XML element with supposed text
        :type element: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.text(element):
            return [self.error.miss_info()]
        messages: list = []
        if self.has_duplicate_blanks(self.text(element)):
            messages.append(self.error.dupl_blanks())
        if not len(self.text(element).split()) >= config["min_word_num"]:
            messages.append(self.error.short())
        return messages if messages else None

    def inspect_unittitle(self, c, level: str) -> list | None:
        """
        Inspect unit title.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["unittitle"][level]["inspect"]:
            return None
        return self.inspect_text(
            c.find("{*}did/{*}unittitle"), self.configuration["unittitle"][level]
        )

    def subordinate_unitdates(self, c) -> dict:
        """
        Get all subordinate unit dates of an component.

        :param c: Component of an EAD record
        :type c: etree._Element
        :return: Dict of unit dates
        :rtype: dict
        """
        dates: dict = {}
        sub_cs: list = c.findall(".//{*}c")
        for sub_c in sub_cs:
            unitid: str = self.text(sub_c.find("{*}did/{*}unitid"))
            if unitid:
                unitdates: list = sub_c.findall("{*}did/{*}unitdate")
                dates[unitid] = self.normalized_unitdates(unitdates)
        return dates

    def normalized_unitdates(self, unitdates: list) -> list:
        """
        Normalize unit dates.

        :param unitdates: List of unit dates
        :type unitdates: list
        :return: List of normalized dates
        :rtype: list
        """
        return [self.normal_date_range(unitdate) for unitdate in unitdates]

    def is_consistent_latest_date(self, sub_date: dict, date: dict) -> bool:
        """
        Check if sub date (latest) is consistent.

        :param sub_date: Sub date of a subordinate component
        :type sub_date: dict
        :param date: Date of the superordinate component
        :type date: dict
        :return: True if date is consistent, False if not
        :rtype: bool
        """
        if sub_date["latest_date"] is None:
            return True
        if (
            date["latest_date"] is not None
            and sub_date["latest_date"] <= date["latest_date"]
        ):
            return True
        return False

    def is_consistent_earliest_date(self, sub_date: dict, date: dict) -> bool:
        """
        Check if sub date (earliest) is consistent.

        :param sub_date: Sub date of a subordinate component
        :type sub_date: dict
        :param date: Date of the superordinate component
        :type date: dict
        :return: True if date is consistent, False if not
        :rtype: bool
        """
        if sub_date["earliest_date"] is None:
            return True
        if (
            date["earliest_date"] is not None
            and sub_date["earliest_date"] >= date["earliest_date"]
        ):
            return True
        return False

    def is_consistent_date(self, sub_date: dict, dates: list) -> bool:
        """
        Check if sub date (earliest and latest) is consistent.

        :param sub_date: Sub date of a subordinate component
        :type sub_date: dict
        :param dates: Dates of the superordinate component
        :type dates: list
        :return: True if date is consistent, False if not
        :rtype: bool
        """
        if sub_date["earliest_date"] is None and sub_date["latest_date"] is None:
            return True
        for date in dates:
            if self.is_consistent_earliest_date(
                sub_date, date
            ) and self.is_consistent_latest_date(sub_date, date):
                return True
        return False

    def inspect_sub_dating(
        self, dates: list, sub_unitid: str, sub_dating: dict
    ) -> list:
        """
        Inspect dates in comparison to sub dates.

        :param dates: Dates of the inspected component
        :type dates: list
        :param sub_unitid: Unit id of subordinate component
        :type sub_unitid: str
        :param sub_dating: Dating of subordinate component
        :type sub_dating: str
        :return: List of error messages
        :rtype: list
        """
        messages: list = []
        for sub_date in sub_dating:
            if not self.is_consistent_date(sub_date, dates):
                messages.append(
                    self.error.inconsistent_date(
                        sub_unitid,
                        f"{sub_date['earliest_date']}/{sub_date['latest_date']}",
                    )
                )
        return messages

    def inspect_unitdates_consistency(self, unitdates: list, c) -> list:
        """
        Inspect consistency of unit dates.

        :param unitdates: Unit dates of the inspected component
        :type unitdates: list
        :param c: Component of an EAD record
        :type c: etree._Element
        :return: List of error messages
        :rtype: list
        """
        messages: list = []
        dates: list = self.normalized_unitdates(unitdates)
        sub_dates: dict = self.subordinate_unitdates(c)
        for unitid, sub_dating in sub_dates.items():
            messages.extend(self.inspect_sub_dating(dates, unitid, sub_dating))
        return messages

    def normal_date_range(self, date) -> dict:
        """
        Get normalized date range.

        :param date: Date to normalize
        :type date: etree._Element
        :return: Normalized date range
        :rtype: dict
        """
        return self.date_range(self.attr(date, "normal"))

    def is_future(self, norm_date: dict) -> bool:
        """
        Check if a date is in the future.

        :param norm_date: Normalized form of the inspected date
        :type norm_date: dict
        :return: True if date is in the future, False if not
        :rtype: bool
        """
        today: date = date.today()
        if (
            norm_date["earliest_date"] is not None
            and norm_date["earliest_date"] > today
        ):
            return True
        if norm_date["latest_date"] is not None and norm_date["latest_date"] > today:
            return True
        return False

    def inspect_date(self, date) -> list:
        """
        Inspect date.

        :param date: Date to inspect
        :type date: etree._Element
        :return: List of error messages
        :rtype: list
        """
        messages: list = []
        norm_date: dict = self.normal_date_range(date)
        if norm_date["earliest_date"] is None or norm_date["latest_date"] is None:
            messages.append(self.error.miss_norm_date(self.text(date)))
        if self.is_future(norm_date):
            messages.append(self.error.future(f"{self.attr(date, 'normal')}"))
        return messages

    def inspect_dates(self, dates: list) -> list:
        """
        Inspect multiple dates.

        :param dates: Dates to inspect
        :type dates: list
        :return: List of error messages
        :rtype: list
        """
        messages: list = []
        for date in dates:
            messages.extend(self.inspect_date(date))
        return messages

    def inspect_unitdates(self, c, level: str) -> list | None:
        """
        Inspect unit dates.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["unitdate"][level]["inspect"]:
            return None
        unitdates: list = c.findall("{*}did/{*}unitdate")
        if not unitdates:
            return [self.error.miss_info()]
        messages: list = []
        messages.extend(self.inspect_dates(unitdates))
        messages.extend(self.inspect_unitdates_consistency(unitdates, c))
        return messages if messages else None

    def inspect_abstract(self, c, level: str) -> list | None:
        """
        Inspect abstract.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["abstract"][level]["inspect"]:
            return None
        return self.inspect_text(
            c.find("{*}did/{*}abstract"), self.configuration["abstract"][level]
        )

    def inspect_genreform(self, c, level: str) -> list | None:
        """
        Inspect genreform.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["genreform"][level]["inspect"]:
            return None
        genreform = c.find("{*}did/{*}physdesc/{*}genreform")
        normal: str = self.attr(genreform, "normal")
        if not normal and not self.text(genreform):
            return [self.error.miss_info()]
        if not normal or normal not in self.configuration["genreform"]["normal"]:
            return [
                self.error.miss_norm_term(normal if normal else self.text(genreform))
            ]
        return None

    def inspect_dimensions(self, c, level: str) -> list | None:
        """
        Inspect dimensions.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["dimensions"][level]["inspect"]:
            return None
        if not self.text(c.find("{*}did/{*}physdesc/{*}dimensions")):
            return [self.error.miss_info()]
        return None

    def inspect_extent(self, c, level: str) -> list | None:
        """
        Inspect extent.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["extent"][level]["inspect"]:
            return None
        if not self.text(c.find("{*}did/{*}physdesc/{*}extent")):
            return [self.error.miss_info()]
        return None

    def inspect_scopecontent(self, c, level: str) -> list | None:
        """
        Inspect scope content.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["scopecontent"][level]["inspect"]:
            return None
        scopecontent_subelems = c.findall("{*}scopecontent/{*}*")
        text: str = " ".join(
            [elem.text.strip() for elem in scopecontent_subelems if self.has_text(elem)]
        )
        return self.inspect_text(
            self.create_element("scopecontent", text),
            self.configuration["scopecontent"][level],
        )

    def inspect_origination(self, origination, level: str) -> list:
        """
        Inspect origination.

        :param origination: XML element of origination
        :type origination: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages
        :rtype: list
        """
        if not self.has_text(origination) and not self.has_subelems(origination):
            return [self.error.empty_elem(origination.tag)]
        if self.has_subelems(origination):
            origination = origination.find("{*}name")
            if not self.has_text(origination):
                return [self.error.empty_elem(origination.tag)]
        return self.inspect_entity(
            self.text(origination),
            self.attr(origination, "authfilenumber"),
            self.configuration["origination"][level],
        )

    def inspect_originations(self, c, level: str) -> list | None:
        """
        Inspect originations.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["origination"][level]["inspect"]:
            return None
        originations = c.findall("{*}did/{*}origination")
        if not originations:
            return [self.error.miss_info()]
        messages: list = []
        for origination in originations:
            messages.extend(self.inspect_origination(origination, level))
        return messages if messages else None

    def inspect_materialspec(self, c, level: str) -> list | None:
        """
        Inspect materialspec.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["materialspec"][level]["inspect"]:
            return None
        if not self.text(c.find("{*}did/{*}materialspec")):
            return [self.error.miss_info()]
        return None

    def inspect_language(self, c, level: str) -> list | None:
        """
        Inspect language.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["language"][level]["inspect"]:
            return None
        language = c.find("{*}did/{*}langmaterial/{*}language")
        if not self.text(language):
            return [self.error.miss_info()]
        if not self.attr(language, "langcode"):
            return [self.error.miss_lang_code()]
        return None

    def inspect_daogrp(self, daogrp) -> list:
        """
        Inspect a digital archival object.

        :param c: Component of an EAD record
        :type c: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list
        """
        if not self.has_subelems(daogrp):
            return [self.error.empty_elem("daogrp")]
        messages: list = []
        if not self.attr(daogrp.find("{*}daoloc"), f"{{{self.xlink_namespace}}}href"):
            messages.append(self.error.miss_link())
        return messages

    def inspect_daos(self, c, level: str) -> list | None:
        """
        Inspect digital archival objects.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["digital_archival_object"][level]["inspect"]:
            return None
        daogrps = c.findall("{*}daogrp")
        if not daogrps:
            return [self.error.miss_info()]
        messages: list = []
        for daogrp in daogrps:
            messages.extend(self.inspect_daogrp(daogrp))
        return messages if messages else None

    def inspect_indexentry(self, indexentry, level: str) -> list:
        """
        Inspect an index entry.

        :param indexentry: XML element of an index entry
        :type indexentry: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(indexentry):
            return [self.error.empty_elem(indexentry.tag)]
        name_subject = indexentry.find("{*}*")
        if not self.has_text(name_subject):
            return [self.error.empty_elem(name_subject.tag)]
        return self.inspect_entity(
            self.text(name_subject),
            self.attr(name_subject, "authfilenumber"),
            self.configuration["index"][level],
        )

    def inspect_index(self, c, level: str) -> list | None:
        """
        Inspect index.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["index"][level]["inspect"]:
            return None
        indexentries: list = c.findall("{*}index/{*}indexentry")
        if not indexentries:
            return [self.error.miss_info()]
        messages: list = []
        for indexentry in indexentries:
            messages.extend(self.inspect_indexentry(indexentry, level))
        if len(indexentries) < self.configuration["index"][level]["min_num"]:
            messages.append(self.error.few())
        return messages if messages else None

    def inspect_userestrict(self, c, level: str) -> list | None:
        """
        Inspect use restrict.

        :param c: Component of an EAD record
        :type c: etree._Element
        :param level: Level of the inspected EAD component
        :type level: str
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["userestrict"][level]["inspect"]:
            return None
        extref = c.find("{*}userestrict[@type='ead']//{*}extref")
        if (
            not self.has_text(extref)
            and not self.has_text(self.rights_ead)
            and not self.attr(extref, f"{{{self.xlink_namespace}}}href")
            and not self.attr(self.rights_ead, f"{{{self.xlink_namespace}}}href")
        ):
            return [self.error.miss_rights("EAD")]
        return None



import pytest
from lxml import etree
from datetime import date
from nfdinspector.ead_inspector import EADInspector


def xml(xml_string):
    return etree.fromstring(xml_string)


class Test_EADInspector:

    def test_configure(self):
        ei = EADInspector()
        default_config = ei.configuration.copy()
        ei.configure(
            {
                "unittitle": {
                    "collection": {"min_word_num": 5},
                    "class": {"inspect": False, "min_word_num": 0},
                }
            }
        )
        assert ei.configuration["unittitle"]["collection"]["min_word_num"] == 5
        assert (
            ei.configuration["unittitle"]["collection"]["inspect"]
            == default_config["unittitle"]["collection"]["inspect"]
        )
        assert ei.configuration["unittitle"]["class"] == {
            "inspect": False,
            "min_word_num": 0,
        }
        assert (
            ei.configuration["unittitle"]["file"] == default_config["unittitle"]["file"]
        )
        assert ei.configuration["abstract"] == default_config["abstract"]
        ei.configuration = default_config.copy()
        ei.configure(
            {
                "unknown": {
                    "collection": {"min_word_num": 5},
                    "class": {"inspect": False, "min_word_num": 0},
                }
            }
        )
        assert ei.configuration == default_config
        ei.configuration = default_config.copy()
        ei.configure(
            {
                "unittitle": {
                    "collection": {"min_word_num": 5},
                    "unknown": {"inspect": False, "min_word_num": 0},
                }
            }
        )
        assert ei.configuration["unittitle"]["collection"]["min_word_num"] == 5
        assert (
            ei.configuration["unittitle"]["class"]
            == default_config["unittitle"]["class"]
        )

    def test_configure_level(self):
        ei = EADInspector()
        default_config = ei.configuration.copy()
        ei.configure_level(
            "unittitle",
            {
                "collection": {"min_word_num": 5},
                "class": {"inspect": 0, "min_word_num": "0"},
            },
        )
        assert ei.configuration["unittitle"]["collection"]["min_word_num"] == 5
        assert (
            ei.configuration["unittitle"]["collection"]["inspect"]
            == default_config["unittitle"]["collection"]["inspect"]
        )
        assert ei.configuration["unittitle"]["class"] == {
            "inspect": False,
            "min_word_num": 0,
        }
        assert (
            ei.configuration["unittitle"]["file"] == default_config["unittitle"]["file"]
        )
        assert ei.configuration["abstract"] == default_config["abstract"]
        ei.configuration = default_config.copy()
        ei.configure_level(
            "unittitle",
            {
                "collection": {"min_word_num": 5},
                "unknown": {"inspect": False, "min_word_num": 0},
            },
        )
        assert ei.configuration["unittitle"]["collection"]["min_word_num"] == 5
        assert (
            ei.configuration["unittitle"]["class"]
            == default_config["unittitle"]["class"]
        )

    def test_configure_setting(self):
        ei = EADInspector()
        default_config = ei.configuration.copy()
        ei.configure_setting("unittitle", "file", {"inspect": False})
        assert ei.configuration["unittitle"]["file"]["inspect"] == False
        ei.configure_setting("unittitle", "file", {"inspect": 0})
        assert ei.configuration["unittitle"]["file"]["inspect"] == False
        assert (
            ei.configuration["unittitle"]["file"]["min_word_num"]
            == default_config["unittitle"]["file"]["min_word_num"]
        )
        ei.configuration = default_config.copy()
        ei.configure_setting("unittitle", "file", {"unknown": 5})
        assert ei.configuration == default_config
        ei.configure_setting("genreform", "normal", ["Akten"])
        assert ei.configuration["genreform"]["normal"] == ["Akten"]
        assert (
            ei.configuration["genreform"]["class"]
            == default_config["genreform"]["class"]
        )

    def test_inspect_id(self):
        ei = EADInspector()
        assert ei.inspect_id(xml("<c id='test'></c>")) == "test"
        assert ei.inspect_id(xml("<c id='test'/>")) == "test"
        assert ei.inspect_id(xml("<c otherid='test'></c>")) == ei.error.miss_info()
        assert ei.inspect_id(xml("<c id=''></c>")) == ei.error.miss_info()
        assert ei.inspect_id(xml("<c></c>")) == ei.error.miss_info()
        assert ei.inspect_id(xml("<c/>")) == ei.error.miss_info()

    def test_inspect_unitid(self):
        ei = EADInspector()
        assert (
            ei.inspect_unitid(xml("<c><did><unitid>test</unitid></did></c>")) == "test"
        )
        assert (
            ei.inspect_unitid(xml("<c><did><unitid></unitid></did></c>"))
            == ei.error.miss_info()
        )
        assert (
            ei.inspect_unitid(xml("<c><did><unitid/></did></c>"))
            == ei.error.miss_info()
        )
        assert ei.inspect_unitid(xml("<c><did></did></c>")) == ei.error.miss_info()

    def test_inspect_text(self):
        ei = EADInspector()
        config = {"inspect": True, "min_word_num": 2}
        assert ei.inspect_text(xml("<elem>test test</elem>"), config) == None
        assert ei.error.miss_info() in ei.inspect_text(xml("<elem></elem>"), config)
        assert ei.error.miss_info() in ei.inspect_text(xml("<elem/>"), config)
        assert ei.error.short() in ei.inspect_text(xml("<elem>test</elem>"), config)
        assert ei.error.dupl_blanks() in ei.inspect_text(
            xml("<elem>test  test</elem>"), config
        )

    def test_inspect_unittitle(self):
        ei = EADInspector()
        ei.configuration["unittitle"]["_"] = {"inspect": True, "min_word_num": 2}
        assert (
            ei.inspect_unittitle(
                xml("<c><did><unittitle>test test</unittitle></did></c>"),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_unittitle(
            xml("<c><did><unittitle></unittitle></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_unittitle(
            xml("<c><did><unittitle/></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_unittitle(
            xml("<c><did></did></c>"),
            "_",
        )
        assert ei.error.short() in ei.inspect_unittitle(
            xml("<c><did><unittitle>test</unittitle></did></c>"),
            "_",
        )
        assert ei.error.dupl_blanks() in ei.inspect_unittitle(
            xml("<c><did><unittitle>test  test</unittitle></did></c>"),
            "_",
        )

    def test_subordinate_unitdates(self):
        ei = EADInspector()
        assert ei.subordinate_unitdates(xml("<c></c>")) == {}
        assert ei.subordinate_unitdates(
            xml("<c><c><did><unitid>1</unitid></did></c></c>")
        ) == {"1": []}
        assert ei.subordinate_unitdates(
            xml(
                "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c></c>"
            )
        ) == {
            "1": [
                {
                    "earliest_date": date(2015, 1, 1),
                    "latest_date": date(2015, 1, 1),
                }
            ]
        }
        assert ei.subordinate_unitdates(
            xml(
                "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/><unitdate normal='2016-01-01'/></did></c></c>"
            )
        ) == {
            "1": [
                {
                    "earliest_date": date(2015, 1, 1),
                    "latest_date": date(2015, 1, 1),
                },
                {
                    "earliest_date": date(2016, 1, 1),
                    "latest_date": date(2016, 1, 1),
                },
            ]
        }
        assert ei.subordinate_unitdates(
            xml(
                "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c><c><did><unitid>2</unitid><unitdate normal='2016-01-01'/></did></c></c>"
            )
        ) == {
            "1": [
                {
                    "earliest_date": date(2015, 1, 1),
                    "latest_date": date(2015, 1, 1),
                }
            ],
            "2": [
                {
                    "earliest_date": date(2016, 1, 1),
                    "latest_date": date(2016, 1, 1),
                }
            ],
        }

    def test_normalized_unitdates(self):
        ei = EADInspector()
        assert ei.normalized_unitdates([xml("<unitdate normal='2015-01-01'/>")]) == [
            {
                "earliest_date": date(2015, 1, 1),
                "latest_date": date(2015, 1, 1),
            }
        ]
        assert ei.normalized_unitdates(
            [xml("<unitdate normal='2015-01-01/2015-01-15'/>")]
        ) == [
            {
                "earliest_date": date(2015, 1, 1),
                "latest_date": date(2015, 1, 15),
            }
        ]
        assert ei.normalized_unitdates(
            [
                xml("<unitdate normal='2015-01-01/2015-01-15'/>"),
                xml("<unitdate normal='2016-01-01/2016-01-15'/>"),
            ]
        ) == [
            {
                "earliest_date": date(2015, 1, 1),
                "latest_date": date(2015, 1, 15),
            },
            {
                "earliest_date": date(2016, 1, 1),
                "latest_date": date(2016, 1, 15),
            },
        ]

    def test_is_consistent_latest_date(self):
        ei = EADInspector()
        assert (
            ei.is_consistent_latest_date(
                {"earliest_date": date(2015, 1, 5), "latest_date": None},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == True
        )
        assert (
            ei.is_consistent_latest_date(
                {"earliest_date": date(2015, 1, 5), "latest_date": date(2015, 1, 13)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == True
        )
        assert (
            ei.is_consistent_latest_date(
                {"earliest_date": date(2014, 1, 5), "latest_date": date(2015, 1, 15)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == True
        )
        assert (
            ei.is_consistent_latest_date(
                {"earliest_date": date(2015, 1, 5), "latest_date": date(2015, 1, 16)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == False
        )
        assert (
            ei.is_consistent_latest_date(
                {"earliest_date": date(2010, 1, 5), "latest_date": date(2016, 1, 16)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == False
        )

    def test_is_consistent_earliest_date(self):
        ei = EADInspector()
        assert (
            ei.is_consistent_earliest_date(
                {"earliest_date": None, "latest_date": date(2015, 1, 10)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == True
        )
        assert (
            ei.is_consistent_earliest_date(
                {"earliest_date": date(2015, 1, 5), "latest_date": date(2015, 1, 10)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == True
        )
        assert (
            ei.is_consistent_earliest_date(
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 20)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == True
        )
        assert (
            ei.is_consistent_earliest_date(
                {"earliest_date": date(2014, 12, 31), "latest_date": date(2015, 1, 10)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == False
        )
        assert (
            ei.is_consistent_earliest_date(
                {"earliest_date": date(2010, 12, 16), "latest_date": date(2018, 1, 10)},
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
            )
            == False
        )

    def test_is_consistent_date(self):
        ei = EADInspector()
        assert (
            ei.is_consistent_date(
                {"earliest_date": date(2015, 1, 5), "latest_date": date(2015, 1, 13)},
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
            )
            == True
        )
        assert (
            ei.is_consistent_date(
                {"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)},
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
            )
            == True
        )
        assert (
            ei.is_consistent_date(
                {"earliest_date": date(2015, 2, 2), "latest_date": date(2015, 2, 2)},
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 15),
                    },
                    {
                        "earliest_date": date(2015, 2, 1),
                        "latest_date": date(2015, 2, 15),
                    },
                ],
            )
            == True
        )
        assert (
            ei.is_consistent_date(
                {"earliest_date": date(2014, 12, 15), "latest_date": date(2015, 1, 13)},
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
            )
            == False
        )
        assert (
            ei.is_consistent_date(
                {"earliest_date": date(2015, 1, 13), "latest_date": date(2015, 1, 22)},
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 15),
                    },
                    {
                        "earliest_date": date(2015, 1, 20),
                        "latest_date": date(2015, 2, 15),
                    },
                ],
            )
            == False
        )

    def test_inspect_sub_dating(self):
        ei = EADInspector()
        assert (
            ei.inspect_sub_dating(
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
                "id",
                [{"earliest_date": date(2015, 1, 5), "latest_date": date(2015, 1, 13)}],
            )
            == []
        )
        assert (
            ei.inspect_sub_dating(
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
                "id",
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 1),
                    },
                    {
                        "earliest_date": date(2015, 1, 2),
                        "latest_date": date(2015, 1, 2),
                    },
                ],
            )
            == []
        )
        assert (
            ei.inspect_sub_dating(
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 15),
                    },
                    {
                        "earliest_date": date(2016, 1, 1),
                        "latest_date": date(2016, 1, 15),
                    },
                ],
                "id",
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 1),
                    },
                    {
                        "earliest_date": date(2015, 1, 2),
                        "latest_date": date(2015, 1, 2),
                    },
                    {
                        "earliest_date": date(2016, 1, 2),
                        "latest_date": date(2016, 1, 2),
                    },
                ],
            )
            == []
        )
        assert (
            ei.inspect_sub_dating(
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
                "id",
                [{"earliest_date": date(2015, 1, 5), "latest_date": date(2015, 1, 17)}],
            )
            != []
        )
        assert (
            ei.inspect_sub_dating(
                [{"earliest_date": date(2015, 1, 1), "latest_date": date(2015, 1, 15)}],
                "id",
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 1),
                    },
                    {
                        "earliest_date": date(2015, 1, 2),
                        "latest_date": date(2015, 2, 2),
                    },
                ],
            )
            != []
        )
        assert (
            ei.inspect_sub_dating(
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 15),
                    },
                    {
                        "earliest_date": date(2016, 1, 1),
                        "latest_date": date(2016, 1, 15),
                    },
                ],
                "id",
                [
                    {
                        "earliest_date": date(2015, 1, 1),
                        "latest_date": date(2015, 1, 1),
                    },
                    {
                        "earliest_date": date(2015, 1, 2),
                        "latest_date": date(2015, 1, 2),
                    },
                    {
                        "earliest_date": date(2015, 1, 2),
                        "latest_date": date(2016, 1, 2),
                    },
                ],
            )
            != []
        )

    def test_inspect_unitdates_consistency(self):
        ei = EADInspector()
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c></c>"
                ),
            )
            == []
        )
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01/2015-01-15'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c><c><did><unitid>2</unitid><unitdate normal='2015-01-15'/></did></c></c>"
                ),
            )
            == []
        )
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01/2015-01-15'/>"),
                    xml("<unitdate normal='2016-01-01/2016-01-15'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c><c><did><unitid>2</unitid><unitdate normal='2016-01-15'/></did></c></c>"
                ),
            )
            == []
        )
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-02'/></did></c></c>"
                ),
            )
            != []
        )
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01/2015-01-15'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c><c><did><unitid>2</unitid><unitdate normal='2016-01-15'/></did></c></c>"
                ),
            )
            != []
        )
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01/2015-01-15'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01/2015-01-30'/></did></c></c>"
                ),
            )
            != []
        )
        assert (
            ei.inspect_unitdates_consistency(
                [
                    xml("<unitdate normal='2015-01-01/2015-01-15'/>"),
                    xml("<unitdate normal='2016-01-01/2016-01-15'/>"),
                ],
                xml(
                    "<c><c><did><unitid>1</unitid><unitdate normal='2015-01-01/2016-01-10'/></did></c></c>"
                ),
            )
            != []
        )

    def test_normal_date_range(self):
        ei = EADInspector()
        assert ei.normal_date_range(xml("<elem normal='2015-01-01/2015-01-15'/>")) == {
            "earliest_date": date(2015, 1, 1),
            "latest_date": date(2015, 1, 15),
        }
        assert ei.normal_date_range(xml("<elem normal='2015-01-01/2015-01-01'/>")) == {
            "earliest_date": date(2015, 1, 1),
            "latest_date": date(2015, 1, 1),
        }
        assert ei.normal_date_range(xml("<elem/>")) == {
            "earliest_date": None,
            "latest_date": None,
        }
        assert ei.normal_date_range(xml("<elem normal='2015-01-01'/>")) == {
            "earliest_date": date(2015, 1, 1),
            "latest_date": date(2015, 1, 1),
        }
        assert ei.normal_date_range(xml("<elem normal='2015-01-01'/>")) != {
            "earliest_date": date(2015, 1, 1),
            "latest_date": None,
        }
        assert ei.normal_date_range(xml("<elem normal='2015-01-01'/>")) != {
            "earliest_date": None,
            "latest_date": date(2015, 1, 1),
        }

    def test_is_future(self):
        ei = EADInspector()
        assert (
            ei.is_future(
                {"earliest_date": date(3000, 1, 1), "latest_date": date(3000, 12, 31)}
            )
            == True
        )
        assert (
            ei.is_future(
                {"earliest_date": date(2000, 1, 1), "latest_date": date(3000, 12, 31)}
            )
            == True
        )
        assert (
            ei.is_future(
                {"earliest_date": date(2000, 1, 1), "latest_date": date(2000, 12, 31)}
            )
            == False
        )

    def test_inspect_date(self):
        ei = EADInspector()
        assert ei.inspect_date(xml("<elem normal='2015-01-01'/>")) == []
        assert ei.inspect_date(xml("<elem normal='2015-01-01/2015-01-15'/>")) == []
        assert ei.error.miss_norm_date("01.01.2015") in ei.inspect_date(
            xml("<elem>01.01.2015</elem>")
        )
        assert ei.error.miss_norm_date("01.01.2015") in ei.inspect_date(
            xml("<elem normal='01.01.2015'>01.01.2015</elem>")
        )
        assert ei.error.miss_norm_date("") in ei.inspect_date(
            xml("<elem normal='01.01.2015'/>")
        )
        assert ei.error.future("3000-01-01/3000-12-31") in ei.inspect_date(
            xml("<elem normal='3000-01-01/3000-12-31'/>")
        )
        assert ei.error.future("3000-01-01") in ei.inspect_date(
            xml("<elem normal='3000-01-01'/>")
        )

    def test_inspect_dates(self):
        ei = EADInspector()
        assert ei.inspect_dates([xml("<elem normal='2015-01-01'/>")]) == []
        assert ei.inspect_dates([xml("<elem normal='2015-01-01/2015-01-15'/>")]) == []
        assert (
            ei.inspect_dates(
                [
                    xml("<elem normal='2015-01-01/2015-01-15'/>"),
                    xml("<elem normal='2016-01-01/2016-01-15'/>"),
                ]
            )
            == []
        )
        assert ei.error.miss_norm_date("01.01.2015") in ei.inspect_dates(
            [xml("<elem>01.01.2015</elem>")]
        )
        assert ei.error.miss_norm_date("01.01.2015") in ei.inspect_dates(
            [xml("<elem normal='01.01.2015'>01.01.2015</elem>")]
        )
        assert ei.error.miss_norm_date("") in ei.inspect_dates(
            [xml("<elem normal='01.01.2015'/>")]
        )
        assert ei.error.miss_norm_date("01.01.2015") in ei.inspect_dates(
            [
                xml("<elem>01.01.2015</elem>"),
                xml("<elem>01.01.2016</elem>"),
            ]
        )
        assert ei.error.miss_norm_date("01.01.2016") not in ei.inspect_dates(
            [
                xml("<elem>01.01.2015</elem>"),
                xml("<elem normal='2016-01-01'>01.01.2016</elem>"),
            ]
        )

    def test_inspect_unitdates(self):
        ei = EADInspector()
        ei.configuration["unitdate"]["_"] = {"inspect": True}
        assert (
            ei.inspect_unitdates(
                xml(
                    "<c><did><unitdate normal='2015-01-01'/></did><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c></c>"
                ),
                "_",
            )
            == None
        )
        assert (
            ei.inspect_unitdates(
                xml(
                    "<c><did><unitdate normal='2015-01-01/2015-01-15'/></did><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c><c><did><unitid>2</unitid><unitdate normal='2015-01-15'/></did></c></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in (
            ei.inspect_unitdates(
                xml("<c><did></did></c>"),
                "_",
            )
        )
        assert ei.error.miss_norm_date("01.01.2015") in (
            ei.inspect_unitdates(
                xml("<c><did><unitdate>01.01.2015</unitdate></did></c>"),
                "_",
            )
        )
        assert ei.error.inconsistent_date("2", "2016-01-15/2016-01-15") in (
            ei.inspect_unitdates(
                xml(
                    "<c><did><unitdate normal='2015-01-01/2015-01-15'/></did><c><did><unitid>1</unitid><unitdate normal='2015-01-01'/></did></c><c><did><unitid>2</unitid><unitdate normal='2016-01-15'/></did></c></c>"
                ),
                "_",
            )
        )
        assert ei.error.inconsistent_date("1", "2015-01-01/2016-01-15") in (
            ei.inspect_unitdates(
                xml(
                    "<c><did><unitdate normal='2015-01-01/2015-01-15'/></did><c><did><unitid>1</unitid><unitdate normal='2015-01-01/2016-01-15'/></did></c></c>"
                ),
                "_",
            )
        )

    def test_inspect_abstract(self):
        ei = EADInspector()
        ei.configuration["abstract"]["_"] = {"inspect": True, "min_word_num": 2}
        assert (
            ei.inspect_abstract(
                xml("<c><did><abstract>test test</abstract></did></c>"),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_abstract(
            xml("<c><did><abstract></abstract></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_abstract(
            xml("<c><did><abstract/></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_abstract(
            xml("<c><did></did></c>"),
            "_",
        )
        assert ei.error.short() in ei.inspect_abstract(
            xml("<c><did><abstract>test</abstract></did></c>"),
            "_",
        )
        assert ei.error.dupl_blanks() in ei.inspect_abstract(
            xml("<c><did><abstract>test  test</abstract></did></c>"),
            "_",
        )

    def inspect_genreform(self):
        ei = EADInspector()
        ei.configuration["genreform"]["_"] = {"inspect": True}
        ei.configuration["genreform"]["normal"] = ["test"]
        assert (
            ei.inspect_genreform(
                xml(
                    "<c><did><physdesc><genreform normal='test'>test</genreform></physdesc></did></c>"
                ),
                "_",
            )
            == None
        )
        assert (
            ei.inspect_genreform(
                xml(
                    "<c><did><physdesc><genreform normal='test'/></physdesc></did></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_genreform(
            xml("<c><did><physdesc><genreform></genreform></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_genreform(
            xml("<c><did><physdesc><genreform/></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_genreform(
            xml("<c><did><physdesc></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_norm_term() in ei.inspect_genreform(
            xml("<c><did><physdesc><genreform>test</genreform></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_norm_term() in ei.inspect_genreform(
            xml(
                "<c><did><physdesc><genreform normal=''>test</genreform></physdesc></did></c>"
            ),
            "_",
        )
        assert ei.error.miss_norm_term() in ei.inspect_genreform(
            xml(
                "<c><did><physdesc><genreform normal='not normal'>test</genreform></physdesc></did></c>"
            ),
            "_",
        )

    def test_inspect_dimensions(self):
        ei = EADInspector()
        ei.configuration["dimensions"]["_"] = {"inspect": True}
        assert (
            ei.inspect_dimensions(
                xml(
                    "<c><did><physdesc><dimensions>test</dimensions></physdesc></did></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_dimensions(
            xml("<c><did><physdesc><dimensions></dimensions></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_dimensions(
            xml("<c><did><physdesc><dimensions/></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_dimensions(
            xml("<c><did><physdesc></physdesc></did></c>"),
            "_",
        )

    def test_inspect_extent(self):
        ei = EADInspector()
        ei.configuration["extent"]["_"] = {"inspect": True}
        assert (
            ei.inspect_extent(
                xml("<c><did><physdesc><extent>test</extent></physdesc></did></c>"),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_extent(
            xml("<c><did><physdesc><extent></extent></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_extent(
            xml("<c><did><physdesc><extent/></physdesc></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_extent(
            xml("<c><did><physdesc></physdesc></did></c>"),
            "_",
        )

    def test_inspect_scopecontent(self):
        ei = EADInspector()
        ei.configuration["scopecontent"]["_"] = {"inspect": True, "min_word_num": 4}
        assert (
            ei.inspect_scopecontent(
                xml(
                    "<c><scopecontent><head>test test</head><p>test test</p></scopecontent></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_scopecontent(
            xml("<c><scopecontent></scopecontent></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_scopecontent(
            xml("<c><scopecontent/></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_scopecontent(
            xml("<c></c>"),
            "_",
        )
        assert ei.error.short() in ei.inspect_scopecontent(
            xml(
                "<c><scopecontent><head>test test</head><p>test</p></scopecontent></c>"
            ),
            "_",
        )
        assert ei.error.dupl_blanks() in ei.inspect_scopecontent(
            xml(
                "<c><scopecontent><head>test  test</head><p>test test</p></scopecontent></c>"
            ),
            "_",
        )

    def test_inspect_origination(self):
        ei = EADInspector()
        ei.configuration["origination"]["_"] = {"inspect": True, "ref": True}
        assert (
            ei.inspect_origination(
                xml("<origination authfilenumber='1'>test</origination>"),
                "_",
            )
            == []
        )
        assert ei.error.empty_elem("origination") in ei.inspect_origination(
            xml("<origination></origination>"),
            "_",
        )
        assert ei.error.empty_elem("origination") in ei.inspect_origination(
            xml("<origination/>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_origination(
            xml("<origination>test</origination>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_origination(
            xml("<origination authfilenumber=''>test</origination>"),
            "_",
        )

    def test_inspect_originations(self):
        ei = EADInspector()
        ei.configuration["origination"]["_"] = {"inspect": True, "ref": True}
        assert (
            ei.inspect_originations(
                xml(
                    "<c><did><origination authfilenumber='1'>test</origination></did></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.empty_elem("origination") in ei.inspect_originations(
            xml("<c><did><origination></origination></did></c>"),
            "_",
        )
        assert ei.error.empty_elem("origination") in ei.inspect_originations(
            xml("<c><did><origination/></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_originations(
            xml("<c><did></did></c>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_originations(
            xml("<c><did><origination authfilenumber=''>test</origination></did></c>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_originations(
            xml("<c><did><origination>test</origination></did></c>"),
            "_",
        )
        assert ei.error.miss_ref("test2") in ei.inspect_originations(
            xml(
                "<c><did><origination>test</origination><origination>test2</origination></did></c>"
            ),
            "_",
        )

    def test_inspect_materialspec(self):
        ei = EADInspector()
        ei.configuration["materialspec"]["_"] = {"inspect": True}
        assert (
            ei.inspect_materialspec(
                xml("<c><did><materialspec>test</materialspec></did></c>"),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_materialspec(
            xml("<c><did><materialspec></materialspec></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_materialspec(
            xml("<c><did><materialspec/></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_extent(
            xml("<c><did></did></c>"),
            "_",
        )

    def test_inspect_language(self):
        ei = EADInspector()
        ei.configuration["language"]["_"] = {"inspect": True}
        assert (
            ei.inspect_language(
                xml(
                    "<c><did><langmaterial><language langcode='ger'>test</language></langmaterial></did></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in ei.inspect_language(
            xml("<c><did><langmaterial><language></language></langmaterial></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_language(
            xml("<c><did><langmaterial><language/></langmaterial></did></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_extent(
            xml("<c><did><langmaterial></langmaterial></did></c>"),
            "_",
        )
        assert ei.error.miss_lang_code() in ei.inspect_language(
            xml(
                "<c><did><langmaterial><language langcode=''>test</language></langmaterial></did></c>"
            ),
            "_",
        )
        assert ei.error.miss_lang_code() in ei.inspect_language(
            xml(
                "<c><did><langmaterial><language>test</language></langmaterial></did></c>"
            ),
            "_",
        )

    def test_inspect_daogrp(self):
        ei = EADInspector()
        wrap = ["<daogrp xmlns:xlink='http://www.w3.org/1999/xlink'>", "</daogrp>"]
        assert (
            ei.inspect_daogrp(xml(f"{wrap[0]}<daoloc xlink:href='url'/>{wrap[1]}"))
            == []
        )
        assert ei.error.empty_elem("daogrp") in (
            ei.inspect_daogrp(xml(f"{wrap[0]}{wrap[1]}"))
        )
        assert ei.error.miss_link() in (
            ei.inspect_daogrp(xml(f"{wrap[0]}<daoloc xlink:href=''/>{wrap[1]}"))
        )
        assert ei.error.miss_link() in (
            ei.inspect_daogrp(xml(f"{wrap[0]}<daoloc/>{wrap[1]}"))
        )

    def test_inspect_daos(self):
        ei = EADInspector()
        wrap = ["<c xmlns:xlink='http://www.w3.org/1999/xlink'>", "</c>"]
        ei.configuration["digital_archival_object"]["_"] = {"inspect": True}
        assert (
            ei.inspect_daos(
                xml(f"{wrap[0]}<daogrp><daoloc xlink:href='url'/></daogrp>{wrap[1]}"),
                "_",
            )
            == None
        )
        assert ei.error.miss_info() in (
            ei.inspect_daos(xml(f"{wrap[0]}{wrap[1]}"), "_")
        )
        assert ei.error.empty_elem("daogrp") in (
            ei.inspect_daos(xml(f"{wrap[0]}<daogrp></daogrp>{wrap[1]}"), "_")
        )
        assert ei.error.empty_elem("daogrp") in (
            ei.inspect_daos(xml(f"{wrap[0]}<daogrp/>{wrap[1]}"), "_")
        )
        assert ei.error.miss_link() in (
            ei.inspect_daos(
                xml(f"{wrap[0]}<daogrp><daoloc xlink:href=''/></daogrp>{wrap[1]}"), "_"
            )
        )
        assert ei.error.miss_link() in (
            ei.inspect_daos(xml(f"{wrap[0]}<daogrp><daoloc/></daogrp>{wrap[1]}"), "_")
        )

    def test_inspect_indexentry(self):
        ei = EADInspector()
        ei.configuration["index"]["_"] = {"inspect": True, "ref": True, "min_num": 2}
        assert (
            ei.inspect_indexentry(
                xml(
                    "<indexentry><subject authfilenumber='1'>test</subject></indexentry>"
                ),
                "_",
            )
            == []
        )
        assert (
            ei.inspect_indexentry(
                xml(
                    "<indexentry><persname authfilenumber='1'>test</persname></indexentry>"
                ),
                "_",
            )
            == []
        )
        assert (
            ei.inspect_indexentry(
                xml(
                    "<indexentry><corpname authfilenumber='1'>test</corpname></indexentry>"
                ),
                "_",
            )
            == []
        )
        assert ei.error.empty_elem("subject") in ei.inspect_indexentry(
            xml("<indexentry><subject></subject></indexentry>"),
            "_",
        )
        assert ei.error.empty_elem("subject") in ei.inspect_indexentry(
            xml("<indexentry><subject/></indexentry>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_indexentry(
            xml("<indexentry><subject>test</subject></indexentry>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_indexentry(
            xml("<indexentry><subject authfilenumber=''>test</subject></indexentry>"),
            "_",
        )

    def test_inspect_index(self):
        ei = EADInspector()
        ei.configuration["index"]["_"] = {"inspect": True, "ref": True, "min_num": 2}
        assert (
            ei.inspect_index(
                xml(
                    "<c><index><indexentry><subject authfilenumber='1'>test</subject></indexentry><indexentry><subject authfilenumber='2'>test2</subject></indexentry></index></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.empty_elem("subject") in ei.inspect_index(
            xml("<c><index><indexentry><subject/></indexentry></index></c>"),
            "_",
        )
        assert ei.error.empty_elem("subject") in ei.inspect_index(
            xml("<c><index><indexentry><subject></subject></indexentry></index></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_index(
            xml("<c><index></index></c>"),
            "_",
        )
        assert ei.error.miss_info() in ei.inspect_index(
            xml("<c><index/></c>"),
            "_",
        )
        assert ei.error.empty_elem("indexentry") in ei.inspect_index(
            xml("<c><index><indexentry/></index></c>"),
            "_",
        )
        assert ei.error.empty_elem("indexentry") in ei.inspect_index(
            xml("<c><index><indexentry></indexentry></index></c>"),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_index(
            xml(
                "<c><index><indexentry><subject authfilenumber=''>test</subject></indexentry></index></c>"
            ),
            "_",
        )
        assert ei.error.miss_ref("test") in ei.inspect_index(
            xml(
                "<c><index><indexentry><subject>test</subject></indexentry></index></c>"
            ),
            "_",
        )
        assert ei.error.miss_ref("test2") in ei.inspect_index(
            xml(
                "<c><index><indexentry><subject>test</subject></indexentry><indexentry><subject>test2</subject></indexentry></index></c>"
            ),
            "_",
        )
        assert ei.error.few() in ei.inspect_index(
            xml(
                "<c><index><indexentry><subject>test</subject></indexentry></index></c>"
            ),
            "_",
        )

    def test_inspect_userestrict(self):
        ei = EADInspector()
        ei.configuration["userestrict"]["_"] = {"inspect": True, "ref": True}
        ei.rights_ead = None
        assert (
            ei.inspect_userestrict(
                xml(
                    "<c xmlns:xlink='http://www.w3.org/1999/xlink'><userestrict type='ead'><p><extref xlink:href='1'>test</extref></p></userestrict></c>"
                ),
                "_",
            )
            == None
        )
        assert (
            ei.inspect_userestrict(
                xml(
                    "<c><userestrict type='ead'><p><extref>test</extref></p></userestrict></c>"
                ),
                "_",
            )
            == None
        )
        assert (
            ei.inspect_userestrict(
                xml(
                    "<c xmlns:xlink='http://www.w3.org/1999/xlink'><userestrict type='ead'><p><extref xlink:href='1'/></p></userestrict></c>"
                ),
                "_",
            )
            == None
        )
        assert ei.error.miss_rights("EAD") in ei.inspect_userestrict(
            xml(
                "<c xmlns:xlink='http://www.w3.org/1999/xlink'><userestrict type='ead'><p><extref xlink:href=''></extref></p></userestrict></c>"
            ),
            "_",
        )
        assert ei.error.miss_rights("EAD") in ei.inspect_userestrict(
            xml(
                "<c xmlns:xlink='http://www.w3.org/1999/xlink'><userestrict type='ead'></userestrict></c>"
            ),
            "_",
        )
        assert ei.error.miss_rights("EAD") in ei.inspect_userestrict(
            xml(
                "<c xmlns:xlink='http://www.w3.org/1999/xlink'><userestrict><p><extref xlink:href='1'>test</extref></p></userestrict></c>"
            ),
            "_",
        )
        ei.rights_ead = xml("<extref>test</extref>")
        assert (
            ei.inspect_userestrict(
                xml("<c></c>"),
                "_",
            )
            == None
        )


if __name__ == "__main__":
    pytest.main()



from .metadata_inspector import MetadataInspector
import json


class LIDOInspector(MetadataInspector):
    """Class for inspectors that examine records in LIDO-XML."""

    def __init__(self, error_lang="en") -> None:
        """
        Construct LIDOInspector with specific error language.

        :param error_lang: Error language for the inspections.
        :type error_lang: str, default 'en'
        """
        super().__init__(error_lang)
        self._lido_namespace: str = "http://www.lido-schema.org"
        self._lido_objects: list = []
        self._configuration: dict = {
            "title": {
                "inspect": True,
                "unique": True,
                "distinct_from_type": True,
                "min_word_num": 2,
                "max_word_num": 20,
            },
            "category": {"inspect": True, "ref": True},
            "object_work_type": {"inspect": True, "ref": True},
            "classification": {"inspect": True, "ref": True},
            "object_description": {
                "inspect": True,
                "unique": True,
                "min_word_num": 20,
                "max_word_num": 500,
            },
            "materials_tech": {
                "inspect": True,
                "ref": True,
                "differentiated": False,
            },
            "object_measurements": {"inspect": True},
            "event": {"inspect": True, "ref": True},
            "subject_concept": {
                "inspect": True,
                "ref": True,
                "min_num": 3,
            },
            "resource": {"inspect": True},
            "record_type": {"inspect": True, "ref": True},
            "repository_name": {"inspect": True, "ref": True},
            "record_source": {"inspect": True, "ref": True},
            "record_rights": {"inspect": True, "ref": True},
            "record_info": {"inspect": True},
        }
        self._duplicate_titles: set = set()
        self._duplicate_descriptions: set = set()

    @property
    def lido_namespace(self) -> str:
        """Get the LIDO namespace when needed for reading attributes."""
        return self._lido_namespace

    @property
    def lido_objects(self) -> list:
        """Get or set the list of LIDO records. These records are examined during the inspection."""
        return self._lido_objects

    @lido_objects.setter
    def lido_objects(self, lido_objects: list) -> None:
        self._lido_objects = lido_objects

    @property
    def configuration(self) -> dict:
        """Get or set the configuration. The inspection is carried out based on the configuration."""
        return self._configuration

    @configuration.setter
    def configuration(self, configuration: dict) -> None:
        self._configuration = configuration

    @property
    def duplicate_titles(self) -> set:
        """Get or set the set of duplicate titles."""
        return self._duplicate_titles

    @duplicate_titles.setter
    def duplicate_titles(self, duplicate_titles: set) -> None:
        self._duplicate_titles = duplicate_titles

    @property
    def duplicate_descriptions(self) -> set:
        """Get or set the set of duplicate descriptions."""
        return self._duplicate_descriptions

    @duplicate_descriptions.setter
    def duplicate_descriptions(self, duplicate_descriptions: set) -> None:
        self._duplicate_descriptions = duplicate_descriptions

    def read_lido(self, xml_str: str) -> None:
        """
        Parse LIDO-XML from a string and assign LIDO records to the inspector.

        :param xml_str: String with LIDO-XML syntax
        :type xml_str: str
        """
        xml_root = MetadataInspector.read_xml(xml_str)
        self.lido_objects = [obj for obj in xml_root.iter("{*}lido")]

    def read_lido_file(self, file_path: str) -> None:
        """
        Parse LIDO-XML from a file and assign LIDO records to the inspector.

        :param file_path: File path to a XML file
        :type file_path: str
        """
        xml_root = MetadataInspector.read_xml_file(file_path)
        self.lido_objects = [obj for obj in xml_root.iter("{*}lido")]

    def read_lido_files(self, files_path: str) -> None:
        """
        Parse LIDO-XML from multiple files in a folder and assign LIDO records to the inspector.

        :param file_path: File path to a XML file
        :type file_path: str
        """
        xml_roots: list = MetadataInspector.read_xml_files(files_path)
        self.lido_objects = [obj for root in xml_roots for obj in root.iter("{*}lido")]

    def configure(self, config: dict) -> None:
        """
        Alter the default configurations of an inspector.

        :param config: Dict of configurations with the syntax of the default configurations
        :type config: dict
        """
        for key in self.configuration.keys():
            if key in config:
                self.configure_setting(key, config[key])

    def configure_setting(self, setting: str, change: dict) -> None:
        """
        Alter a specific setting in the configurations of an inspector.

        :param setting: Name of the setting to change
        :type setting: str
        :param change: Desired change for the specific setting
        :type change: dict
        """
        for key, value in change.items():
            if key not in self.configuration[setting]:
                continue
            if key in [
                "inspect",
                "unique",
                "distinct_from_type",
                "ref",
                "differentiated",
            ]:
                self.configuration[setting][key] = bool(value)
            elif key in ["min_word_num", "max_word_num", "min_num"]:
                self.configuration[setting][key] = int(value)

    def config_file(self, file_path: str) -> None:
        """
        Read a configuration file and alter the default configurations of an inspector.

        :param file_path: File path to a JSON file with configurations in the required syntax
        :type file_path: str
        """
        with open(file_path, "r") as f:
            config = json.load(f)
        self.configure(config)

    def inspect(self) -> None:
        """Carry out an inspection based on the read-in LIDO records."""
        self.inspections = []
        if self.configuration["title"]["unique"]:
            self.duplicate_titles = self.find_duplicate_titles()
        if self.configuration["object_description"]["unique"]:
            self.duplicate_descriptions = self.find_duplicate_descriptions()
        for lido_object in self.lido_objects:
            inspection: dict = {}
            inspection["lidoRecID"] = self.inspect_lido_rec_id(lido_object)
            inspection["workID"] = self.inspect_work_id(lido_object)
            inspection["title"] = self.inspect_title(lido_object)
            inspection["category"] = self.inspect_category(lido_object)
            inspection["objectWorkType"] = self.inspect_object_work_types(lido_object)
            inspection["classification"] = self.inspect_classifications(lido_object)
            inspection["objectDescription"] = self.inspect_object_description(
                lido_object
            )
            inspection["materialsTech"] = self.inspect_materials_tech(lido_object)
            inspection["objectMeasurements"] = self.inspect_object_measurements(
                lido_object
            )
            inspection["event"] = self.inspect_events(lido_object)
            inspection["subjectConcept"] = self.inspect_subject_concepts(lido_object)
            inspection["resourceSet"] = self.inspect_resource_sets(lido_object)
            inspection["recordType"] = self.inspect_record_type(lido_object)
            inspection["repositoryName"] = self.inspect_repository_name(lido_object)
            inspection["recordSource"] = self.inspect_record_sources(lido_object)
            inspection["recordRights"] = self.inspect_record_rights(lido_object)
            inspection["recordInfoSet"] = self.inspect_record_info_set(lido_object)
            self.inspections.append(inspection)

    def find_duplicates(self, xpath: str) -> set:
        """
        Find duplicates based on an XPATH expression.

        :param xpath: XPATH expression
        :type xpath: str
        :return: All duplicate titles in lido_objects
        :rtype: set
        """
        texts: list = [
            self.text(lido_object.find(xpath)) for lido_object in self.lido_objects
        ]
        seen: set = set()
        return set(
            text for text in texts if text != "" and text in seen or seen.add(text)
        )

    def find_duplicate_titles(self) -> set:
        """
        Find duplicate titles.

        :return: All duplicate titles in lido_objects
        :rtype: set
        """
        return self.find_duplicates(
            "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}titleWrap/{*}titleSet/{*}appellationValue"
        )

    def find_duplicate_descriptions(self) -> set:
        """
        Find duplicate descriptions.

        :return: All duplicate descriptions in lido_objects
        :rtype: set
        """
        return self.find_duplicates(
            "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}objectDescriptionWrap/{*}objectDescriptionSet/{*}descriptiveNoteValue"
        )

    def inspect_lido_rec_id(self, lido_object) -> str:
        """
        Inspect record ID.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: Record ID or error message if missing
        :rtype: str
        """
        lido_rec_id = lido_object.find("{*}lidoRecID")
        return lido_rec_id.text if self.text(lido_rec_id) else self.error.miss_info()

    def inspect_work_id(self, lido_object) -> str:
        """
        Inspect work ID.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: Work ID or error message if missing
        :rtype: str
        """
        work_id = lido_object.find(
            "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}repositoryWrap/{*}repositorySet/{*}workID"
        )
        return work_id.text if self.text(work_id) else self.error.miss_info()

    def is_distinct_from_type(self, lido_object, value: str) -> bool:
        """
        Check if title is distinct from object/work type.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :param value: Text value of the inspected record and element
        :type value: str
        :return: True if title is distinct from object/work type, False if not
        :rtype: bool
        """
        object_work_type = lido_object.find(
            "{*}descriptiveMetadata/{*}objectClassificationWrap/{*}objectWorkTypeWrap/{*}objectWorkType"
        )
        if self.has_subelems(object_work_type):
            return False if value == self.term(object_work_type) else True
        return True

    def inspect_title(self, lido_object) -> list | None:
        """
        Inspect title.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["title"]["inspect"]:
            return None
        return self.inspect_text(
            lido_object.find(
                "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}titleWrap/{*}titleSet"
            ),
            lido_object,
            self.configuration["title"],
        )

    def value(self, parent) -> str:
        """
        Get value (appellation or descriptiveNote) of a text field.

        :param parent: Parent element of the supposed value element.
        :type parent: etree._Element
        :return: Text value
        :rtype: str
        """
        if not self.has_subelems(parent):
            return ""
        value = parent.find(".//{*}appellationValue")
        return (
            value.text
            if self.text(value)
            else self.text(parent.find("{*}descriptiveNoteValue"))
        )

    def term(self, parent) -> str:
        """
        Get term or prefLabel of a concept.

        :param parent: Parent element of the supposed term or prefLabel element.
        :type parent: etree._Element
        :return: Term or label
        :rtype: str
        """
        if not self.has_subelems(parent):
            return ""
        term = parent.find("{*}term")
        return (
            term.text if self.text(term) else self.text(parent.find(".//{*}prefLabel"))
        )

    def is_uniq(self, text: str, element) -> bool:
        """
        Check if title or object description is unique.

        :param text: Text that is checked
        :type text: str
        :param element: XML element with supposed text
        :type element: etree._Element
        :return: True if title/description is unique, False if not
        :rtype: bool
        """
        if (
            (
                element.tag == f"{{{self.lido_namespace}}}titleSet"
                or element.tag == "titleSet"
            )
            and text in self.duplicate_titles
        ) or (
            (
                element.tag == f"{{{self.lido_namespace}}}objectDescriptionSet"
                or element.tag == "objectDescriptionSet"
            )
            and text in self.duplicate_descriptions
        ):
            return False
        return True

    def inspect_text(self, element, lido_object, config: dict) -> list | None:
        """
        Inspect a text element.

        :param element: XML element with supposed text
        :type element: etree._Element
        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.has_subelems(element):
            return [self.error.miss_info()]
        value: str = self.value(element)
        if not value:
            return [self.error.miss_info()]
        messages: list = []
        if self.has_duplicate_blanks(value):
            messages.append(self.error.dupl_blanks())
        if config["unique"]:
            if not self.is_uniq(value, element):
                messages.append(self.error.not_uniq())
        if config.get("distinct_from_type", False):
            if not self.is_distinct_from_type(lido_object, value):
                messages.append(self.error.dist("objectWorkType"))
        if not len(value.split()) >= config["min_word_num"]:
            messages.append(self.error.short())
        if not len(value.split()) <= config["max_word_num"]:
            messages.append(self.error.long())
        return messages if messages else None

    def about(self, element) -> str:
        """
        Get value from about attribute.

        :param element: XML element with supposed about attribute
        :type element: etree._Element
        :return: Value of about
        :rtype: str
        """
        return self.attr(element, f"{{{self.rdf_namespace}}}about")

    def concept_id(self, parent) -> str:
        """
        Get ID of a concept.

        :param parent: Parent element of the supposed conceptID or Concept element.
        :type parent: etree._Element
        :return: conceptID or Concept
        :rtype: str
        """
        if not self.has_subelems(parent):
            return ""
        concept_id = parent.find("{*}conceptID")
        return (
            concept_id.text
            if self.text(concept_id)
            else self.about(parent.find("{*}Concept"))
        )

    def inspect_concept(self, concept, config: dict) -> list:
        """
        Inspect concept.

        :param concept: XML element of the concept
        :type concept: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(concept):
            return [self.error.miss_info()]
        return self.inspect_entity(self.term(concept), self.concept_id(concept), config)

    def inspect_concepts(self, concept_list: list, config: dict) -> list | None:
        """
        Inspect multiple concepts.

        :param concept_list: List of XML elements of concepts
        :type concept_list: list
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not concept_list:
            return [self.error.miss_info()]
        messages: list = []
        for element in concept_list:
            messages.extend(self.inspect_concept(element, config))
        if "min_num" in config:
            if len(concept_list) < config["min_num"]:
                messages.append(self.error.few())
        return messages if messages else None

    def inspect_category(self, lido_object) -> list | None:
        """
        Inspect category.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["category"]["inspect"]:
            return None
        return self.inspect_concepts(
            lido_object.findall("{*}category"), self.configuration["category"]
        )

    def inspect_object_work_types(self, lido_object) -> list | None:
        """
        Inspect object/work types.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["object_work_type"]["inspect"]:
            return None
        return self.inspect_concepts(
            lido_object.findall(
                "{*}descriptiveMetadata/{*}objectClassificationWrap/{*}objectWorkTypeWrap/{*}objectWorkType"
            ),
            self.configuration["object_work_type"],
        )

    def inspect_classifications(self, lido_object) -> list | None:
        """
        Inspect classifications.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["classification"]["inspect"]:
            return None
        return self.inspect_concepts(
            lido_object.findall(
                "{*}descriptiveMetadata/{*}objectClassificationWrap/{*}classificationWrap/{*}classification"
            ),
            self.configuration["classification"],
        )

    def inspect_object_description(self, lido_object) -> list | None:
        """
        Inspect object description.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["object_description"]["inspect"]:
            return None
        return self.inspect_text(
            lido_object.find(
                "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}objectDescriptionWrap/{*}objectDescriptionSet"
            ),
            lido_object,
            self.configuration["object_description"],
        )

    def lido_type(self, element) -> str:
        """
        Get value from type attribute.

        :param element: XML element with supposed type attribute
        :type element: etree._Element
        :return: Value of type
        :rtype: str
        """
        return self.attr(element, f"{{{self.lido_namespace}}}type")

    def has_valid_type(self, elements: list, valid_types: list) -> bool:
        """
        Check if elements have valid types.

        :param elements: Elements with supposed type attributes
        :type elements: list
        :return: True if type is valid, False if not
        :rtype: bool
        """
        for element in elements:
            if self.lido_type(element).lower() in valid_types:
                return True
        return False

    def has_material(self, materials_tech: list) -> bool:
        """
        Check if record contains information about material.

        :param materials_tech: XML elements of materials/techniques
        :type materials_tech: list
        :return: True if record contains information about material, False if not
        :rtype: bool
        """
        return self.has_valid_type(
            materials_tech,
            [
                "http://terminology.lido-schema.org/lido00132",
                "material",
                "materials",
                "substances",
            ],
        )

    def has_tech(self, materials_tech: list) -> bool:
        """
        Check if record contains information about technique.

        :param materials_tech: XML elements of materials/techniques
        :type materials_tech: list
        :return: True if record contains information about technique, False if not
        :rtype: bool
        """
        return self.has_valid_type(
            materials_tech,
            [
                "http://terminology.lido-schema.org/lido00131",
                "technique",
                "methods",
                "technique",
                "technik",
                "entstehungsmethode",
            ],
        )

    def inspect_materials_tech(self, lido_object) -> list | None:
        """
        Inspect materials and techniques.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["materials_tech"]["inspect"]:
            return None
        materials_tech = lido_object.findall(".//{*}materialsTech/{*}termMaterialsTech")
        concepts_inspection = self.inspect_concepts(
            materials_tech, self.configuration["materials_tech"]
        )
        messages: list = []
        if concepts_inspection is not None:
            messages.extend(concepts_inspection)
        if (
            not self.configuration["materials_tech"]["differentiated"]
            or self.error.miss_info() in messages
        ):
            return messages if messages else None
        if not self.has_material(materials_tech):
            messages.append(self.error.miss_mat())
        if not self.has_tech(materials_tech):
            messages.append(self.error.miss_tech())
        return messages if messages else None

    def meas_type(self, measurement_type) -> str:
        """
        Get text or ID of measurement type.

        :param measurement_type: XML element of measurement type.
        :type measurement_type: etree._Element
        :return: Text or ID
        :rtype: str
        """
        return (
            measurement_type.text
            if self.text(measurement_type)
            else self.concept_id(measurement_type)
        )

    def inspect_measurements_set(self, measurements_set) -> list:
        """
        Inspect a measurements set.

        :param measurements_set: XML element of supposed measurements set
        :type measurements_set: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(measurements_set):
            return [self.error.empty_elem("measurementsSet")]
        measurement_type: str = self.meas_type(
            measurements_set.find("{*}measurementType")
        )
        measurement_unit = measurements_set.find("{*}measurementUnit")
        measurement_value = measurements_set.find("{*}measurementValue")
        messages: list = []
        if not measurement_type:
            messages.append(self.error.miss_meas_type())
        if not self.text(measurement_unit) and not self.has_subelems(measurement_unit):
            messages.append(self.error.miss_meas_unit(measurement_type))
        if not self.text(measurement_value):
            messages.append(self.error.miss_meas_value(measurement_type))
        return messages

    def inspect_object_measurements(self, lido_object) -> list | None:
        """
        Inspect objects measurements.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["object_measurements"]["inspect"]:
            return None
        measurements_sets = lido_object.findall(
            "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}objectMeasurementsWrap/{*}objectMeasurementsSet/{*}objectMeasurements/{*}measurementsSet"
        )
        if not measurements_sets:
            return [self.error.miss_info()]
        messages: list = []
        for measurements_set in measurements_sets:
            messages.extend(self.inspect_measurements_set(measurements_set))
        return messages if messages else None

    def inspect_event_type(self, event_type) -> list:
        """
        Inspect event type.

        :param event_type: XML element of supposed event type
        :type event_type: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(event_type):
            return [self.error.miss_event_type()]
        return self.inspect_concept(event_type, self.configuration["event"])

    def place_id(self, parent) -> str:
        """
        Get ID of a place.

        :param parent: Parent element of the supposed place element.
        :type parent: etree._Element
        :return: placeID
        :rtype: str
        """
        return self.text(parent.find("{*}placeID")) if self.has_subelems(parent) else ""

    def inspect_place(self, place, event_type, config: dict) -> list:
        """
        Inspect place.

        :param place: XML element of supposed place
        :type place: etree._Element
        :param event_type: XML element of corresponding event type
        :type event_type: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(place):
            return [self.error.miss_place(self.term(event_type))]
        return self.inspect_entity(self.value(place), self.place_id(place), config)

    def inspect_places(self, places: list, event_type, config: dict) -> list:
        """
        Inspect multiple places.

        :param places: XML elements of supposed places
        :type places: etree._Element
        :param event_type: XML element of corresponding event type
        :type event_type: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages
        :rtype: list
        """
        if self.term(event_type) in ["Event (non-specified)"]:
            return []
        if not places:
            return [self.error.miss_place(self.term(event_type))]
        messages: list = []
        for place in places:
            messages.extend(self.inspect_place(place, event_type, config))
        return messages

    def actor_id(self, parent) -> str:
        """
        Get ID of an actor.

        :param parent: Parent element of the supposed actor element.
        :type parent: etree._Element
        :return: actorID
        :rtype: str
        """
        return self.text(parent.find("{*}actorID")) if self.has_subelems(parent) else ""

    def inspect_actor(self, actor, event_type, config: dict) -> list:
        """
        Inspect actor.

        :param actor: XML element of supposed actor
        :type actor: etree._Element
        :param event_type: XML element of corresponding event type
        :type event_type: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(actor):
            return [self.error.miss_actor(self.term(event_type))]
        return self.inspect_entity(self.value(actor), self.actor_id(actor), config)

    def inspect_actors(self, actors: list, event_type, config: dict) -> list:
        """
        Inspect multiple actors.

        :param actors: XML elements of supposed actors
        :type actors: etree._Element
        :param event_type: XML element of corresponding event type
        :type event_type: etree._Element
        :param config: Configuration of the specific inspection.
        :type config: dict
        :return: List of error messages
        :rtype: list
        """
        if self.term(event_type) in ["Event (non-specified)"]:
            return []
        if not actors:
            return [self.error.miss_actor(self.term(event_type))]
        messages: list = []
        for actor in actors:
            messages.extend(self.inspect_actor(actor, event_type, config))
        return messages

    def inspect_date(self, date, event_type) -> list:
        """
        Inspect date.

        :param date: XML element of supposed date
        :type date: etree._Element
        :param event_type: XML element of corresponding event type
        :type event_type: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if self.term(event_type) in ["Event (non-specified)"]:
            return []
        if not self.has_subelems(date):
            return [self.error.miss_date(self.term(event_type))]
        messages: list = []
        if not self.text(date.find("{*}earliestDate")):
            messages.append(self.error.miss_earl_date(self.term(event_type)))
        if not self.text(date.find("{*}latestDate")):
            messages.append(self.error.miss_lat_date(self.term(event_type)))
        return messages

    def summarize_event_messages(self, messages: list, event_type: str) -> list:
        """
        Summarize several event-specific error messages (missing actor, place and date).

        :param messages: Error messages of an event
        :type messages: list
        :param event_type: XML element of corresponding event type
        :type event_type: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if (
            self.error.miss_actor(event_type) in messages
            and self.error.miss_place(event_type) in messages
            and self.error.miss_date(event_type) in messages
        ):
            messages.append(self.error.miss_event_info(event_type))
        return [
            message
            for message in messages
            if message != self.error.miss_actor(event_type)
            and message != self.error.miss_place(event_type)
            and message != self.error.miss_date(event_type)
        ]

    def inspect_event(self, event) -> list:
        """
        Inspect event.

        :param event: XML element of supposed event
        :type event: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(event):
            return [self.error.empty_elem("event")]
        messages: list = []
        event_type = event.find("{*}eventType")
        messages.extend(self.inspect_event_type(event_type))
        messages.extend(
            self.inspect_actors(
                event.findall("{*}eventActor/{*}actorInRole/{*}actor"),
                event_type,
                self.configuration["event"],
            )
        )
        messages.extend(
            self.inspect_places(
                event.findall("{*}eventPlace/{*}place"),
                event_type,
                self.configuration["event"],
            )
        )
        messages.extend(
            self.inspect_date(event.find("{*}eventDate/{*}date"), event_type)
        )
        return self.summarize_event_messages(messages, self.term(event_type))

    def inspect_events(self, lido_object) -> list | None:
        """
        Inspect events.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["event"]["inspect"]:
            return None
        events: list = lido_object.findall(
            "{*}descriptiveMetadata/{*}eventWrap/{*}eventSet/{*}event"
        )
        if not events:
            return [self.error.miss_info()]
        messages: list = []
        for event in events:
            messages.extend(self.inspect_event(event))
        return messages if messages else None

    def inspect_subject_concepts(self, lido_object) -> list | None:
        """
        Inspect subject concepts.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["subject_concept"]["inspect"]:
            return None
        return self.inspect_concepts(
            lido_object.findall(
                "{*}descriptiveMetadata/{*}objectRelationWrap/{*}subjectWrap/{*}subjectSet/{*}subject/{*}subjectConcept"
            ),
            self.configuration["subject_concept"],
        )

    def inspect_resource_set(self, resource_set) -> list:
        """
        Inspect resource set.

        :param resource_set: XML element of supposed resource set
        :type resource_set: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(resource_set):
            return [self.error.empty_elem("resourceSet")]
        messages: list = []
        link_resource: str = self.text(
            resource_set.find("{*}resourceRepresentation/{*}linkResource")
        )
        if not link_resource:
            messages.append(self.error.miss_link())
        if not self.text(resource_set.find("{*}rightsResource/{*}rightsType/{*}term")):
            messages.append(self.error.miss_rights(link_resource))
        if not self.text(resource_set.find("{*}resourceType/{*}term")):
            messages.append(self.error.miss_res_type(link_resource))
        return messages

    def inspect_resource_sets(self, lido_object) -> list | None:
        """
        Inspect resource sets.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["resource"]["inspect"]:
            return None
        resource_sets = lido_object.findall(
            "{*}administrativeMetadata/{*}resourceWrap/{*}resourceSet"
        )
        if not resource_sets:
            return [self.error.miss_info()]
        messages: list = []
        for resource_set in resource_sets:
            messages.extend(self.inspect_resource_set(resource_set))
        return messages if messages else None

    def inspect_record_type(self, lido_object) -> list | None:
        """
        Inspect record type.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["record_type"]["inspect"]:
            return None
        messages: list = self.inspect_concept(
            lido_object.find("{*}administrativeMetadata/{*}recordWrap/{*}recordType"),
            self.configuration["record_type"],
        )
        return messages if messages else None

    def legal_body_id(self, parent) -> str:
        """
        Get ID of a legal body.

        :param parent: Parent element of the supposed legal body element.
        :type parent: etree._Element
        :return: legalBodyID
        :rtype: str
        """
        return (
            self.text(parent.find("{*}legalBodyID"))
            if self.has_subelems(parent)
            else ""
        )

    def inspect_repository_name(self, lido_object) -> list | None:
        """
        Inspect repository name.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["repository_name"]["inspect"]:
            return None
        repository_name = lido_object.find(
            "{*}descriptiveMetadata/{*}objectIdentificationWrap/{*}repositoryWrap/{*}repositorySet/{*}repositoryName"
        )
        if not self.has_subelems(repository_name):
            return [self.error.miss_info()]
        messages: list = []
        messages.extend(
            self.inspect_entity(
                self.value(repository_name),
                self.legal_body_id(repository_name),
                self.configuration["repository_name"],
            )
        )
        return messages if messages else None

    def inspect_record_source(self, record_source) -> list:
        """
        Inspect record source.

        :param record_source: XML element of supposed record source
        :type record_source: etree._Element
        :return: List of error messages
        :rtype: list
        """
        if not self.has_subelems(record_source):
            return [self.error.empty_elem("recordSource")]
        return self.inspect_entity(
            self.value(record_source),
            self.legal_body_id(record_source),
            self.configuration["record_source"],
        )

    def inspect_record_sources(self, lido_object) -> list | None:
        """
        Inspect record sources.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["record_source"]["inspect"]:
            return None
        record_sources = lido_object.findall(
            "{*}administrativeMetadata/{*}recordWrap/{*}recordSource"
        )
        if not record_sources:
            return [self.error.miss_info()]
        messages: list = []
        for record_source in record_sources:
            messages.extend(self.inspect_record_source(record_source))
        return messages if messages else None

    def inspect_record_rights(self, lido_object) -> list | None:
        """
        Inspect record rights.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["record_rights"]["inspect"]:
            return None
        return self.inspect_concepts(
            lido_object.findall(
                "{*}administrativeMetadata/{*}recordWrap/{*}recordRights/{*}rightsType"
            ),
            self.configuration["record_rights"],
        )

    def inspect_record_info_set(self, lido_object) -> list | None:
        """
        Inspect record information.

        :param lido_object: Record of an object in LIDO-XML
        :type lido_object: etree._Element
        :return: List of error messages, None if there are no errors
        :rtype: list | None
        """
        if not self.configuration["record_info"]["inspect"]:
            return None
        record_info_set = lido_object.find(
            "{*}administrativeMetadata/{*}recordWrap/{*}recordInfoSet"
        )
        if not self.has_subelems(record_info_set):
            return [self.error.miss_info()]
        messages: list = []
        if not self.has_text(record_info_set.find("{*}recordInfoLink")):
            messages.append(self.error.miss_link())
        if not self.has_text(record_info_set.find("{*}recordMetadataDate")):
            messages.append(
                self.error.miss_date("http://terminology.lido-schema.org/lido00472")
            )
        return messages if messages else None



class Error:
    """Class with various error messages for the metadata inspections"""

    def __init__(self, language: str) -> None:
        """
        Construct Error with specific language.

        :param language: Error language for the inspections.
        :type language: str
        """
        self._language: str = language

    @property
    def language(self) -> str:
        """Get and set the language for the error messages."""
        return self._language

    @language.setter
    def language(self, language) -> None:
        self._language = language

    def miss_info(self) -> str:
        """
        Get error message for missing information.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing information"
            case "de":
                return "Angabe fehlt"
            case _:
                return "missing information"

    def empty_elem(self, tag: str) -> str:
        """
        Get error message for empty XML element.

        :param tag: Tag of the concerned element
        :type tag: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"empty element ({tag})"
            case "de":
                return f"leeres Element ({tag})"
            case _:
                return f"empty element ({tag})"

    def miss_label(self, id: str) -> str:
        """
        Get error message for missing label.

        :param id: ID of the concerned entity
        :type id: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing label ({id})"
            case "de":
                return f"Bezeichnung fehlt ({id})"
            case _:
                return f"missing label ({id})"

    def miss_ref(self, label: str) -> str:
        """
        Get error message for missing reference/ID.

        :param label: Label of the concerned entity
        :type label: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing reference/ID ({label})"
            case "de":
                return f"Verweis/ID fehlt ({label})"
            case _:
                return f"missing reference/ID ({label})"

    def not_uniq(self) -> str:
        """
        Get error message for text that is not unique.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"not unique"
            case "de":
                return f"nicht einzigartig"
            case _:
                return f"not unique"

    def dupl_text(self) -> str:
        """
        Get error message for duplicate text.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"duplicate text"
            case "de":
                return f"duplizierter Text"
            case _:
                return f"duplicate text"

    def dist(self, compare: str) -> str:
        """
        Get error message for missing distinction.

        :param compare: Comparison
        :type compare: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"not distinct from {compare}"
            case "de":
                return f"kein Unterschied zu {compare}"
            case _:
                return f"not distinct from {compare}"

    def short(self) -> str:
        """
        Get error message for shortness.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "too short"
            case "de":
                return "zu kurz"
            case _:
                return "too short"

    def long(self) -> str:
        """
        Get error message for length.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "too long"
            case "de":
                return "zu lang"
            case _:
                return "too long"

    def miss_mat(self) -> str:
        """
        Get error message for missing explicit material.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing explicit material"
            case "de":
                return "explizites Material fehlt"
            case _:
                return "missing explicit material"

    def miss_tech(self) -> str:
        """
        Get error message for missing explicit technique.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing explicit technique"
            case "de":
                return "explizite Technik fehlt"
            case _:
                return "missing explicit technique"

    def miss_meas_type(self) -> str:
        """
        Get error message for missing measurement type.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing measurement type"
            case "de":
                return "Messgröße fehlt"
            case _:
                return "missing measurement type"

    def miss_meas_unit(self, meas_type: str) -> str:
        """
        Get error message for missing measurement unit.

        :param meas_type: Measurement type
        :type meas_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing measurement unit ({meas_type})"
            case "de":
                return f"Maßeinheit fehlt ({meas_type})"
            case _:
                return f"missing measurement unit ({meas_type})"

    def miss_meas_value(self, meas_type: str) -> str:
        """
        Get error message for missing measurement value.

        :param meas_type: Measurement type
        :type meas_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing measurement value ({meas_type})"
            case "de":
                return f"Messwert fehlt ({meas_type})"
            case _:
                return f"missing measurement value ({meas_type})"

    def miss_event_type(self) -> str:
        """
        Get error message for missing event type.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing event type"
            case "de":
                return "Eventtyp fehlt"
            case _:
                return "missing event type"

    def miss_event_info(self, event_type: str) -> str:
        """
        Get error message for missing event info.

        :param event_type: Event type
        :type event_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing actor, place or date ({event_type})"
            case "de":
                return f"Akteur:in, Ort oder Datierung fehlen ({event_type})"
            case _:
                return f"missing actor, place or date ({event_type})"

    def miss_actor(self, event_type: str) -> str:
        """
        Get error message for missing actor.

        :param event_type: Event type
        :type event_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing actor ({event_type})"
            case "de":
                return f"Akteur:in fehlt ({event_type})"
            case _:
                return f"missing actor ({event_type})"

    def miss_place(self, event_type: str) -> str:
        """
        Get error message for missing place.

        :param event_type: Event type
        :type event_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing place ({event_type})"
            case "de":
                return f"Ort fehlt ({event_type})"
            case _:
                return f"missing place ({event_type})"

    def future(self, date: str) -> str:
        """
        Get error message for date in future.

        :param date: Date string
        :type date: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"date in future ({date})"
            case "de":
                return f"Datum liegt in der Zukunft ({date})"
            case _:
                return f"date in future ({date})"

    def miss_date(self, event_type: str) -> str:
        """
        Get error message for missing date.

        :param event_type: Event type
        :type event_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing date ({event_type})"
            case "de":
                return f"Datierung fehlt ({event_type})"
            case _:
                return f"missing date ({event_type})"

    def miss_norm_date(self, text_date: str) -> str:
        """
        Get error message for missing normalized date.

        :param text_date: Date as text
        :type text_date: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing normalized (ISO-8601) date ({text_date})"
            case "de":
                return f"normalisierte (ISO-8601) Datierung fehlt ({text_date})"
            case _:
                return f"missing normalized (ISO-8601) date ({text_date})"

    def miss_earl_date(self, event_type: str) -> str:
        """
        Get error message for missing earliest date.

        :param event_type: Event type
        :type event_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing earliest date ({event_type})"
            case "de":
                return f"Anfangsdatum fehlt ({event_type})"
            case _:
                return f"missing earliest date ({event_type})"

    def miss_lat_date(self, event_type: str) -> str:
        """
        Get error message for missing latest date.

        :param event_type: Event type
        :type event_type: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing latest date ({event_type})"
            case "de":
                return f"Enddatum fehlt ({event_type})"
            case _:
                return f"missing latest date ({event_type})"

    def miss_norm_term(self, term: str) -> str:
        """
        Get error message for missing normalized term.

        :param term: Term that is not normalized
        :type term: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing normalized term ({term})"
            case "de":
                return f"normalisierter Begriff fehlt ({term})"
            case _:
                return f"missing normalized term ({term})"

    def few(self) -> str:
        """
        Get error message for too few entries.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "too few entries"
            case "de":
                return "zu wenige Einträge"
            case _:
                return "too few entries"

    def miss_link(self) -> str:
        """
        Get error message for missing link.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing link"
            case "de":
                return "Link fehlt"
            case _:
                return "missing link"

    def miss_lang_code(self) -> str:
        """
        Get error message for missing language code.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "missing language code"
            case "de":
                return "Sprachcode fehlt"
            case _:
                return "missing language code"

    def miss_rights(self, add: str) -> str:
        """
        Get error message for missing rights statement.

        :param add: Additional information
        :type add: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing rights statement ({add})"
            case "de":
                return f"Rechteangabe fehlt ({add})"
            case _:
                return f"missing rights statement ({add})"

    def miss_res_type(self, add: str) -> str:
        """
        Get error message for missing resource type.

        :param add: Additional information
        :type add: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"missing resource type ({add})"
            case "de":
                return f"Ressourcentyp fehlt ({add})"
            case _:
                return f"missing resource type ({add})"

    def dupl_blanks(self) -> str:
        """
        Get error message for duplicate blanks.

        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return "duplicate blanks"
            case "de":
                return "doppelte Leerzeichen"
            case _:
                return "duplicate blanks"

    def inconsistent_date(self, id: str, inconsistency: str) -> str:
        """
        Get error message for missing inconsistent date.

        :param id: ID of concerned file
        :type id: str
        :param inconsistency: Inconsistent date
        :type inconsistency: str
        :return: Error message
        :rtype: str
        """
        match self.language:
            case "en":
                return f"inconsistent date ({id}: {inconsistency})"
            case "de":
                return f"inkonsistente Datierung ({id}: {inconsistency})"
            case _:
                return f"inconsistent date ({id}: {inconsistency})"
